#!/bin/bash

# Initial packager: Filippo "NeverBorn" Mesirca (filippo.mesirca@gmail.com)
# Maintainer: Son|c (emmanuel.bailleul@cegetel.net)
# Sources location : 
# - http://mir2.ovh.net/ftp.apache.org/dist/httpd/httpd-2.2.14.tar.bz2
# - http://apache.cict.fr/httpd/patches/apply_to_2.2.14/CVE-2009-3555-2.2.patch
# ChangeLog for version 2.2.6 :
# - exit all extra config files ; one just needs buildfile to create
# pkg, including layout
# - mod_proxy et al now included by default in pkg
# * 2.2.9 :
#   - added source file creation code
#   - added 'checkconf' to rc script
# * 2.2.9-54.2 :
#   - patched for CVE-2008-2939 (mod_proxy_ftp xss attack)
# * 2.2.11 :
#   - More modules, including ldap
# * 2.2.13 :
#   - Security fixes
#   - First version supporting SNI
# * 2.2.14 :
#   - TLS Security fix (CVE-2009-3555)
#   - Added logrotate conf file

# program name/version
softversion='2.2.14'
softname='apache2'

# architecture/package version
arch='i486'
cpu='i686'
packageversion='64.1'

# number of parallel tasks of make
ntasks='-j2'

# parameters needed for the build process
buildir=$(pwd)
package="$softname-$softversion-$arch-$packageversion"

# prepare the build result directory
dest="$buildir/$package"
rm -rf $dest
rm -f $package.*
mkdir -p $dest
mkdir -p $dest/install
mkdir -p $dest/usr/src/$softname-$softversion

# make a slack-desc file
cat <<EOF > $dest/install/slack-desc
       |-----handy-ruler------------------------------------------------------|
apache2: apache2 (Web server)
apache2:
apache2: Apache is an HTTP server designed as a plug-in replacement for the
apache2: NCSA HTTP server. It fixies numerous bugs in the NCSA server and
apache2: includes many frequently requested new features, and has an API which
apache2: allows it to be extended to meet users' needs more easily.
apache2: Apache is the most popular web server in the known universe; over
apache2: half of the servers on the Internet are running Apache or one of
apache2: its variants.
apache2:
apache2: 
EOF

# extract sources
srcdir="httpd-$softversion"
cd $buildir
rm -rf $srcdir
tar jxvf $srcdir.tar.bz2
cd $srcdir

# Patch for CVE-2009-3555
patch -p0 < ../CVE-2009-3555-2.2.patch

# set correct permissions
chown -R root.root . 
find . -perm 777 -exec chmod 755 {} \;
find . -perm 775 -exec chmod 755 {} \;
find . -perm 666 -exec chmod 644 {} \;
find . -perm 664 -exec chmod 644 {} \;

# configure package for compilation
 
echo "
#   Layout for ZenWalk Linux 
<Layout ZenWalk>
    prefix:        /usr
    exec_prefix:   \${prefix}
    bindir:        \${exec_prefix}/bin
    sbindir:       \${exec_prefix}/sbin
    libdir:        \${exec_prefix}/lib
    libexecdir:    \${exec_prefix}/libexec/apache
    mandir:        \${prefix}/man
    sysconfdir:    /etc/apache
    datadir:       /var/www
    installbuilddir: \${datadir}/build
    errordir:      \${datadir}/error
    iconsdir:      \${datadir}/icons
    htdocsdir:     \${datadir}/htdocs
    manualdir:     \${datadir}/manual
    cgidir:        \${datadir}/cgi-bin
    includedir:    \${prefix}/include/apache
    localstatedir: /var
    runtimedir:    \${localstatedir}/run
    logfiledir:    \${localstatedir}/log/apache
    proxycachedir: \${localstatedir}/proxy
</Layout>
" >> config.layout

#zcat $buildir/config.layout.zenwalk.gz >> config.layout
export CFLAGS="-O2 -march=$arch -mtune=$cpu"
export CXXFLAGS="-O2 -march=$arch -mtune=$cpu"
./configure \
   --enable-layout=ZenWalk \
   --enable-mods-shared="all authn_alias ssl proxy headers rewrite ldap authnz_ldap \
      deflate" \
   --with-ssl=/usr \
   --with-ldap

# compile package
make $ntasks

# install package to the target directory
make install DESTDIR=$dest

# copy the doc files
mkdir -p $dest/usr/doc/$softname-$softversion
cp -a \
   ABOUT_APACHE CHANGES INSTALL LAYOUT LICENSE NOTICE README* ROADMAP VERSIONING \
   $dest/usr/doc/$softname-$softversion

# rename apache configuration files
( cd $dest/etc/apache
  for file in * ; do
    if [ -f $file ]; then
      mv ${file} ${file}.default
    fi
  done

echo "
#Uncomment this to enable mod_php
#include /etc/apache/mod_php.conf" >> httpd.conf.default

cd $dest/etc/apache/extra
  for file in * ; do
    mv ${file} ${file}.default
  done
  # we have the .default files: we don't need this folder anymore
  rm -rf $dest/etc/apache/original
)

# put the initialization script in $dest/etc/rc.d/
mkdir -p $dest/etc/rc.d

echo "#!/bin/sh
#
# /etc/rc.d/rc.httpd
# Start/stop/restart the Apache web server.
#
# To make Apache start automatically at boot, make this
# file executable: chmod 755 /etc/rc.d/rc.httpd
#

case \"\$1\" in
   'start')
      /usr/sbin/apachectl start ;;
   'stop')
      /usr/sbin/apachectl stop ;;
   'restart')
      /usr/sbin/apachectl restart ;;
   'checkconf')
      /usr/sbin/apachectl -t ;;
   *)
      echo \"usage \$0 start|stop|restart|checkconf\" ;;
esac" > $dest/etc/rc.d/rc.httpd.new

chmod 644 $dest/etc/rc.d/rc.httpd.new
chown root.root $dest/etc/rc.d/rc.httpd.new

# strip executables 
( cd $dest
   find . | xargs file | grep "executable" | grep ELF | cut -f 1 -d : | xargs strip --strip-unneeded 2> /dev/null
   find . | xargs file | grep "shared object" | grep ELF | cut -f 1 -d : | xargs strip --strip-unneeded 2> /dev/null
)
  
# gzip man pages and info files
gzip -9 $dest/usr/man/*/*

# put the ZenWalk build and other needed files in the right directory
cp -a $buildir/build-$softname.sh $dest/usr/src/$softname-$softversion
chown -R root.root $dest/usr/src/$softname-$softversion

# change files ownerships for the $dest/usr/bin and $dest/usr/sbin
chown -R root.root $dest/usr/bin
chown -R root.root $dest/usr/sbin

# create a logrotate conf file
mkdir $dest/etc/logrotate.d
echo "/var/log/apache/*log {
        daily
        rotate 30
        compress
        missingok
        copytruncate
} " > $dest/etc/logrotate.d/apache2.new

# create doinst.sh file
echo "config() {
  NEW=\"\$1.new\"
  OLD=\"\$1\"
  # If there's no config file by that name, mv it over:
  if [ ! -r \$OLD ]; then
    mv \$NEW \$OLD
  elif [ \"\`cat \$OLD | md5sum\`\" = \"\`cat \$NEW | md5sum\`\" ]; then # toss the redundant copy
    rm \$NEW
  fi
  # Otherwise, we leave the .new copy for netpkg to consider...
}
config etc/rc.d/rc.httpd
config etc/logrotate.d/apache2

# Create the config files using the default ones, if they don't exist
(cd etc/apache/
for conffile in *.default ; do
  if [ -f \$conffile ]; then
    if [ ! -f \`basename \$conffile .default\` ]; then
      cp -a \$conffile \`basename \$conffile .default\`
    fi
  fi
  # Otherwise, $conffile is not a regular file, and nothing will be done...
  # this is a really strange condition, but, just to be sure not to break
  # things I've added this check
done
cd extra/
for conffile in *.default ; do
  if [ -f \$conffile ]; then
    if [ ! -f \`basename \$conffile .default\` ]; then
      cp -a \$conffile \`basename \$conffile .default\`
    fi
  fi
done)

" > $dest/install/doinst.sh

chmod 755 $dest/install/doinst.sh



#------------------( Create The *.dep File )
cd $dest
depfinder $dest > $buildir/$package.dep

cd $dest
makepkg -l y -c n $dest.txz
#mv $package.tgz $buildir

# computes the md5sum signature for the package
cd $buildir
md5sum $package.txz > $package.md5

# Create source file
echo "http://mir2.ovh.net/ftp.apache.org/dist/httpd/httpd-$softversion.tar.bz2" > $package.src
echo "http://apache.cict.fr/httpd/patches/apply_to_2.2.14/CVE-2009-3555-2.2.patch" >> $package.src
echo "http://download.zenwalk.org/people/emmanuel/$softname-$softversion/build-$softname.sh" >> $package.src

if [ "$1" == "--cleanup" ]; then
  rm -rf $dest
  rm -rf $srcdir
fi



